@@ -1,11 +1,7 @@
-# TODO
-# 4. Document the tests of the new functionality.
-# 5. Document the tests comparing the new build to the old for the old methods.
-# 6. Test gbm.more with t-dist (ensure Misc gets used properly).
 
 .onAttach <- function(lib, pkg)
 {
-   library.dynam("gbm", pkg, lib)
+#   library.dynam("gbm", pkg, lib)
 #   require(survival)
 #   require(splines)
 #   require(lattice)
@@ -91,68 +87,6 @@ print.gbm <- function(x, ... )
    cat( "There were", length( x$var.names ), "predictors of which",
        sum( ri > 0 ), "had non-zero influence.\n" )
 
-   if (x$distribution$name != "pairwise" && (!is.null(x$data)))
-   {
-      d <- reconstructGBMdata( x )
-      if (x$distribution$name == "multinomial")
-      {
-         n.class <- x$num.classes
-
-         yn <- as.numeric( d[, x$response.name ] )
-
-         p <- predict( x, n.trees=best , type = "response", newdata=d )
-         p <- apply( p, 1, function( x , labels ){ labels[ x == max( x ) ] },
-                    labels = colnames( p ))
-         p <- as.numeric(as.factor( p ))
-         r <- yn
-
-         conf.mat <- matrix(table(c(r + n.class * p, (n.class + (1:(n.class^2))))),
-                            nrow = n.class)
-         conf.mat <- conf.mat - 1
-
-         pred.acc <- round(100 * sum(diag(conf.mat)) / sum(conf.mat),2)
-
-         conf.mat <- cbind(conf.mat, round(100*diag(conf.mat)/rowSums(conf.mat),2))
-         dimnames(conf.mat) <- list(x$classes, c(x$classes, "Pred. Acc."))
-
-         cat("\nConfusion matrix:\n")
-         print(conf.mat)
-
-         cat("\nPrediction Accuracy = ", pred.acc, "%\n", sep = "")
-      }
-      else if (x$distribution$name %in% c("bernoulli", "adaboost", "huberized"))
-      {
-         p <- predict( x , newdata=d, n.tree=best , type = "response")
-         p <- ifelse( p < .5, 0, 1 )
-
-         conf.mat <- matrix( table( c( d[,x$response.name] + 2 * p , 0:3 )), ncol=2 )
-         conf.mat <- conf.mat - 1
-
-         pred.acc <- round(100 * sum(diag(conf.mat)) / sum(conf.mat),2)
-
-         conf.mat <- cbind(conf.mat,  round(100*diag(conf.mat)/rowSums(conf.mat),2))
-         dimnames(conf.mat) <- list(c("0","1"), c("0", "1", "Pred. Acc."))
-
-         cat("\nConfusion matrix:\n")
-         print(conf.mat)
-
-         cat("\nPrediction Accuracy = ", pred.acc, "%\n", sep = "")
-      }
-      else if (x$distribution$name %in%
-               c("gaussian", "laplace", "poisson", "quantile", "tdist" ) )
-      {
-         r <- d[, 1] - predict( x, type="response", newdata=d, n.tree=best )
-         if ( x$distribution$name == "poisson" )
-         {
-            cat( "Summary of response residuals:\n" )
-         }
-         else {
-            cat( "Summary of residuals:\n" )
-         }
-         print( quantile( r ) )
-         cat( "\n" )
-      }
-   }
    invisible()
 }
 
@@ -790,6 +724,7 @@ gbm.more <- function(object,
          group        <- group[ord.group]
          y            <- y[ord.group]
          x            <- x[ord.group,,drop=FALSE]
+         w            <- x[ord.group]
          object$fit   <- object$fit[ord.group] # object$fit is stored in the original order
 
          # Split into train and validation set, at group boundary
@@ -878,7 +813,7 @@ gbm.more <- function(object,
                     n.classes = as.integer(object$num.classes),
                     shrinkage = as.double(object$shrinkage),
                     bag.fraction = as.double(object$bag.fraction),
-                    nTrain = as.integer(nTrain),
+                    train.fraction = as.integer(nTrain),
                     fit.old = as.double(object$fit),
                     n.cat.splits.old = as.integer(length(object$c.splits)),
                     n.trees.old = as.integer(object$n.trees),
@@ -957,16 +892,45 @@ gbm.fit <- function(x,y,
                     n.minobsinnode = 10,
                     shrinkage = 0.001,
                     bag.fraction = 0.5,
-                    nTrain,
+                    nTrain = NULL,
+                    train.fraction = NULL,
                     keep.data = TRUE,
                     verbose = TRUE,
                     var.names = NULL,
                     response.name = NULL,
                     group = NULL)
 {
+
+   if(is.character(distribution))
+   {
+      distribution <- list(name=distribution)
+   }
+
    cRows <- nrow(x)
    cCols <- ncol(x)
 
+   # the preferred way to specify the number of training instances is via parameter 'nTrain'.
+   # parameter 'train.fraction' is only maintained for backward compatibility.
+
+   if(!is.null(nTrain) && !is.null(train.fraction))
+   {
+      stop("Parameters 'nTrain' and 'train.fraction' cannot both be specified")
+   }
+   else if(!is.null(train.fraction))
+   {
+      warning("Parameter 'train.fraction' of gbm.fit is deprecated, please specify 'nTrain' instead")
+      nTrain <- floor(train.fraction*cRows)
+   } else if(is.null(nTrain))
+   {
+     # both undefined, use all training data
+     nTrain <- cRows
+   }
+
+   if (is.null(train.fraction)){
+      train.fraction <- nTrain / cRows
+   }
+
+
    if(is.null(var.names))
    {
       if(is.matrix(x))
@@ -997,7 +961,7 @@ gbm.fit <- function(x,y,
    # check dataset size
    if(nTrain * bag.fraction <= 2*n.minobsinnode+1)
    {
-      stop("The dataset size is too small or subsampling rate is too large: cRows*train.fraction*bag.fraction <= n.minobsinnode")
+      stop("The dataset size is too small or subsampling rate is too large: nTrain*bag.fraction <= n.minobsinnode")
    }
 
    if(nrow(x) != ifelse(class(y)=="Surv", nrow(y), length(y)))
@@ -1077,7 +1041,10 @@ gbm.fit <- function(x,y,
 
    nClass <- 1
 
-   if(is.character(distribution)) distribution <- list(name=distribution)
+   if(is.character(distribution))
+   {
+      distribution <- list(name=distribution)
+   }
    if(!("name" %in% names(distribution)))
    {
       stop("The distribution is missing a 'name' component, for example list(name=\"gaussian\")")
@@ -1315,6 +1282,7 @@ gbm.fit <- function(x,y,
    gbm.obj$num.classes <- nClass
    gbm.obj$n.trees <- length(gbm.obj$trees) / nClass
    gbm.obj$nTrain <- nTrain
+   gbm.obj$train.fraction <- train.fraction
    gbm.obj$response.name <- response.name
    gbm.obj$shrinkage <- shrinkage
    gbm.obj$var.levels <- var.levels
@@ -1497,6 +1465,7 @@ gbm <- function(formula = formula(data),
       group        <- group[ord.group]
       y            <- y[ord.group]
       x            <- x[ord.group,,drop=FALSE]
+      w            <- w[ord.group]
 
       # Split into train and validation set, at group boundary
       num.groups.train <- max(1, round(train.fraction * nlevels(group)))
